/*
 *                                 jStock
 *                    (C) Copyright 2003, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

package com.exponto.jstock.indicators;
import com.exponto.jstock.LogDataSeries;
import com.exponto.jstock.LogTradingData;
import com.exponto.util.*;
import java.util.ArrayList;
import org.apache.log4j.Logger;
/**
 * Computes the RSI values for a LogDataSeries. 
 * 
 * Updates the rsiparameters with start and stop dates for the current series. 
 * 
 * @author ttranu
 *
 */
public class RsiAlgorithm {
	
	static Logger logger = Logger.getLogger(RsiAlgorithm.class);
		
	public RsiAlgorithm() {
		
	}
	/**
	 * Note: Sources on the internet does not agree on how to compute RSI
	 * The formula itself is not the problem, it is:
	 * 	RSI = 100 - [100/(1+RS)] where...
	 *		RS = Average of up day's closes / Average of down day's closes
	 *
	 *It's the RS part that is the problem - does avarage of up day closes 
	 *mean sum of positive changes divided by number of days with positive
	 *changes or divided by the whole periode? 
	 *
	 *It seems that historically the RS part has been computed in a 
	 *special way. The first value of RSI was computed with division of the 
	 *whole peridode. The  next value was computed by multiplying the 
	 *previous value with periode -1, then adding/subtracting  the new price
	 *change and then divide again by the whole periode. By this way a 
	 *price change once used in the calculation is never completly forgotten.
	 *
	 * By this way the resulting RSI will vary depending on where you start
	 * your calculation.   
	 *
	 *I have (tried to) implement both, methodes are called 
	 *findRSINewWay and findRSIOldWay respectivly.
	 *
	 *A second note: This is my interpretation of several source found on the 
	 *internet, among them:
	 *	-	ETF Authority Educational Archive -- THE RELATIVE STRENGTH INDEX (RSI)
	 *			http://www.streetauthority.com/terms/technicalanalysis/rsi.asp
	 *	-	The Investment FAQ is a collection of frequently asked questions and
	 *		answers about investments and personal finance.  This is a plain-text
	 *		version of The Investment FAQ, part 17 of 20.  The web site
	 * 	always has the latest version, including in-line links. Please browse
	 *			http://invest-faq.com/
	 */
	
	public static ArrayList findRSINewWay(LogDataSeries logDS, RsiParameters rsiParam) {
		// "Modern way"
		LogDataSeries lDS =logDS;
		Double zero = new Double(0);
		int numberOfDays = rsiParam.getNumberOfDays();
		FifoQueue delayed = new FifoQueue(numberOfDays, zero);
//		FifoQueue downDays = new FifoQueue(numberOfDays);
		ArrayList rsiList = new ArrayList();
		LogTradingData lTD;
		double current, next, delta, past, rsi;
		int upDays = 0;
		int downDays = 0;
		double sumUp = 0.0;
		double sumDown = 0.0;

		Double temp;
		// Set the periode to the whole lDS
		lDS.setCurrentPeriode();
		lTD = lDS.getNextLTD();
		next = lTD.getClose();
		while (lDS.hasNextLTD()) {
			// May use double[] iso. ArrayList
			current = next;
			lTD = lDS.getNextLTD();
			next = lTD.getClose();
			delta = next - current;
			Double d = new Double(delta);
			temp = (Double) delayed.fetchAndStore(d);
			past = temp.doubleValue();
			if (delta > 0) {
//				"Up" day	
				sumUp = sumUp + delta;
				upDays ++;
			} else if (delta < 0) {
//				"Down" day				
				sumDown = sumDown - delta;
				downDays ++;
			}
			if (past > 0) {
//				"Up" day	
				sumUp = sumUp - past;
				upDays --;
			} else if (past < 0){
//				"Down" day
				sumDown = sumDown + past;
				downDays --;
			}
			rsi = 100 - (100 / (1 + (sumUp / upDays) / (sumDown / downDays)));
			logger.debug("current & next close, rsi: " + current + ", " + next + ", " + rsi);
			rsiList.add(new Double(rsi));			
		}
		rsiParam.setValidTo(lTD.getDate());
		return rsiList;
	}
/**
 * Note that for days with out trading the RSI value of the previous day is stored
 * 
 * @param lDS
 * @return an ArrayList with the RSI values
 */

	public static ArrayList findRSIClassicWay(LogDataSeries lDS, RsiParameters rsiParam){
		// "Historical way"
		//LogDataSeries lDS =logDS;
//		if(lDS.logStock.size()< 500) {
//			logger.debug("rsi: series size" + lDS.logStock.size());
//		}
		int periode = rsiParam.getNumberOfDays();
		ArrayList rsiList = new ArrayList();
		LogTradingData lTD;
		double current, next, delta, past, rsi;
		double sumUp = 0.0;
		double sumDown = 0.0;
		lDS.setCurrentPeriode();
		lTD = lDS.getNextLTD();
		next = lTD.getClose();
		for (int i =1; i <= periode; i++) {
			// To find first "complete" value
			current = next;
			lTD = lDS.getNextLTD();
			if(lTD.isTraded()) {
				next = lTD.getClose();
				delta = next - current;
				if (delta > 0) {
					sumUp = sumUp + delta;
				} else {
					sumDown = sumDown - delta;
				}
			} else {
				i--;
			}
		}
		sumUp = sumUp/periode;
		sumDown = sumDown/periode;
		rsi = 100*sumUp/(sumUp+sumDown);
		//rsi = 100 - (100 / (1 + (sumUp / periode) / (sumDown / periode))); 
		rsiList.add(new Double(rsi));
		rsiParam.setValidFrom(lTD.getDate());
		while (lDS.hasNextLTD()) {
			current = next;
			lTD = lDS.getNextLTD();
			if(lTD.isTraded()) {
				next = lTD.getClose();
				delta = next - current;
				if (delta > 0) {
//					"Up" day	
					sumUp = (((periode -1) *sumUp) + delta)/periode;
					sumDown = (((periode -1) *sumDown))/periode;
				} else if (delta < 0) {
//					"Down" day				
					sumUp = (((periode -1) *sumUp))/periode;
					sumDown = (((periode -1) *sumDown) - delta)/periode;
				}
				rsi = 100*sumUp/(sumUp+sumDown);
//				rsi = 100 - (100 / (1 + (sumUp / periode) / (sumDown / periode)));
				//logger.debug("delta, sumUp, sumDown, rsi (old way): " + delta + ", " + sumUp + ", " + sumDown + ", " + rsi);
			}
			// TODO: Validate that using the previous value is the best way to handle days without trading
			rsiList.add(new Double(rsi));			
		}
		rsiParam.setValidTo(lTD.getDate());
		return rsiList;
	}
}
